@@ -2537,15 +2537,15 @@ out:
     return ret;
 }
 
-static void
-flush_netdev_flows_in_related(struct netdev *netdev, struct netdev *related)
+static int
+netdev_offload_dpdk_flow_flush(struct netdev *netdev)
 {
-    unsigned int tid = netdev_offload_thread_id();
-    struct cmap *map = offload_data_map(related);
+    struct cmap *map = offload_data_map(netdev);
     struct ufid_to_rte_flow_data *data;
+    unsigned int tid = netdev_offload_thread_id();
 
     if (!map) {
-        return;
+        return -1;
     }
 
     CMAP_FOR_EACH (data, node, map) {
@@ -2556,31 +2556,6 @@ flush_netdev_flows_in_related(struct netdev *netdev, struct netdev *related)
             netdev_offload_dpdk_flow_destroy(data);
         }
     }
-}
-
-static bool
-flush_in_vport_cb(struct netdev *vport,
-                  odp_port_t odp_port OVS_UNUSED,
-                  void *aux)
-{
-    struct netdev *netdev = aux;
-
-    /* Only vports are related to physical devices. */
-    if (netdev_vport_is_vport_class(vport->netdev_class)) {
-        flush_netdev_flows_in_related(netdev, vport);
-    }
-
-    return false;
-}
-
-static int
-netdev_offload_dpdk_flow_flush(struct netdev *netdev)
-{
-    flush_netdev_flows_in_related(netdev, netdev);
-
-    if (!netdev_vport_is_vport_class(netdev->netdev_class)) {
-        netdev_ports_traverse(netdev->dpif_type, flush_in_vport_cb, netdev);
-    }
 
     return 0;
 }
@@ -2697,7 +2672,7 @@ netdev_offload_dpdk_hw_miss_packet_recover(struct netdev *netdev,
     if (rte_restore_info.flags & RTE_FLOW_RESTORE_INFO_ENCAPSULATED) {
         if (!vport_netdev->netdev_class ||
             !vport_netdev->netdev_class->pop_header) {
-            VLOG_ERR_RL(&rl, "vport nedtdev=%s with no pop_header method",
+            VLOG_ERR_RL(&rl, "vport netdev=%s with no pop_header method",
                         netdev_get_name(vport_netdev));
             ret = EOPNOTSUPP;
             goto close_vport_netdev;
