@@ -94,6 +94,7 @@ static unixctl_cb_func ovsdb_server_get_active_ovsdb_server;
 static unixctl_cb_func ovsdb_server_connect_active_ovsdb_server;
 static unixctl_cb_func ovsdb_server_disconnect_active_ovsdb_server;
 static unixctl_cb_func ovsdb_server_set_active_ovsdb_server_probe_interval;
+static unixctl_cb_func ovsdb_server_set_relay_source_interval;
 static unixctl_cb_func ovsdb_server_set_sync_exclude_tables;
 static unixctl_cb_func ovsdb_server_get_sync_exclude_tables;
 static unixctl_cb_func ovsdb_server_get_sync_status;
@@ -107,6 +108,7 @@ struct server_config {
     char **sync_exclude;
     bool *is_backup;
     int *replication_probe_interval;
+    int *relay_source_probe_interval;
     struct ovsdb_jsonrpc_server *jsonrpc;
 };
 static unixctl_cb_func ovsdb_server_add_remote;
@@ -328,6 +330,7 @@ main(int argc, char *argv[])
     struct shash all_dbs;
     struct shash_node *node;
     int replication_probe_interval = REPLICATION_DEFAULT_PROBE_INTERVAL;
+    int relay_source_probe_interval = RELAY_SOURCE_DEFAULT_PROBE_INTERVAL;
 
     ovs_cmdl_proctitle_init(argc, argv);
     set_program_name(argv[0]);
@@ -341,7 +344,7 @@ main(int argc, char *argv[])
                   &run_command, &sync_from, &sync_exclude, &active);
     is_backup = sync_from && !active;
 
-    daemon_become_new_user(false);
+    daemon_become_new_user(false, false);
 
     /* Create and initialize 'config_tmpfile' as a temporary file to hold
      * ovsdb-server's most basic configuration, and then save our initial
@@ -359,7 +362,7 @@ main(int argc, char *argv[])
     save_config__(config_tmpfile, &remotes, &db_filenames, sync_from,
                   sync_exclude, is_backup);
 
-    daemonize_start(false);
+    daemonize_start(false, false);
 
     /* Load the saved config. */
     load_config(config_tmpfile, &remotes, &db_filenames, &sync_from,
@@ -377,6 +380,7 @@ main(int argc, char *argv[])
     server_config.sync_exclude = &sync_exclude;
     server_config.is_backup = &is_backup;
     server_config.replication_probe_interval = &replication_probe_interval;
+    server_config.relay_source_probe_interval = &relay_source_probe_interval;
 
     perf_counters_init();
 
@@ -472,6 +476,9 @@ main(int argc, char *argv[])
     unixctl_command_register(
         "ovsdb-server/set-active-ovsdb-server-probe-interval", "", 1, 1,
         ovsdb_server_set_active_ovsdb_server_probe_interval, &server_config);
+    unixctl_command_register(
+        "ovsdb-server/set-relay-source-probe-interval", "", 1, 1,
+        ovsdb_server_set_relay_source_interval, &server_config);
     unixctl_command_register("ovsdb-server/set-sync-exclude-tables", "",
                              0, 1, ovsdb_server_set_sync_exclude_tables,
                              &server_config);
@@ -574,7 +581,9 @@ close_db(struct server_config *config, struct db *db, char *comment)
 }
 
 static struct ovsdb_error * OVS_WARN_UNUSED_RESULT
-update_schema(struct ovsdb *db, const struct ovsdb_schema *schema,
+update_schema(struct ovsdb *db,
+              const struct ovsdb_schema *schema,
+              const struct uuid *txnid,
               bool conversion_with_no_data, void *aux)
 {
     struct server_config *config = aux;
@@ -591,11 +600,17 @@ update_schema(struct ovsdb *db, const struct ovsdb_schema *schema,
         struct ovsdb *new_db = NULL;
         struct ovsdb_error *error;
 
-        error = ovsdb_convert(db, schema, &new_db);
-        if (error) {
-            /* Should never happen, because conversion should have been
-             * checked before writing the schema to the storage. */
-            return error;
+        /* If conversion was triggered by the current process, we might
+         * already have converted version of a database. */
+        new_db = ovsdb_trigger_find_and_steal_converted_db(db, txnid);
+        if (!new_db) {
+            /* No luck.  Converting. */
+            error = ovsdb_convert(db, schema, &new_db);
+            if (error) {
+                /* Should never happen, because conversion should have been
+                 * checked before writing the schema to the storage. */
+                return error;
+            }
         }
         ovsdb_replace(db, new_db);
     } else {
@@ -635,7 +650,7 @@ parse_txn(struct server_config *config, struct db *db,
             return error;
         }
 
-        error = update_schema(db->db, schema, txn_json == NULL, config);
+        error = update_schema(db->db, schema, txnid, txn_json == NULL, config);
         if (error) {
             return error;
         }
@@ -790,7 +805,8 @@ open_db(struct server_config *config, const char *filename)
     add_db(config, db);
 
     if (is_relay) {
-        ovsdb_relay_add_db(db->db, relay_remotes, update_schema, config);
+        ovsdb_relay_add_db(db->db, relay_remotes, update_schema, config,
+                           *config->relay_source_probe_interval);
     }
     return NULL;
 }
@@ -1473,6 +1489,26 @@ ovsdb_server_set_active_ovsdb_server_probe_interval(struct unixctl_conn *conn,
     }
 }
 
+static void
+ovsdb_server_set_relay_source_interval(struct unixctl_conn *conn,
+                                       int argc OVS_UNUSED,
+                                       const char *argv[],
+                                       void *config_)
+{
+    struct server_config *config = config_;
+    int probe_interval;
+
+    if (str_to_int(argv[1], 10, &probe_interval)) {
+        *config->relay_source_probe_interval = probe_interval;
+        save_config(config);
+        ovsdb_relay_set_probe_interval(probe_interval);
+        unixctl_command_reply(conn, NULL);
+    } else {
+        unixctl_command_reply_error(
+            conn, "Invalid probe interval, integer value expected");
+    }
+}
+
 static void
 ovsdb_server_set_sync_exclude_tables(struct unixctl_conn *conn,
                                      int argc OVS_UNUSED,
@@ -1972,6 +2008,7 @@ parse_options(int argc, char *argv[],
         OPT_ACTIVE,
         OPT_NO_DBS,
         OPT_FILE_COLUMN_DIFF,
+        OPT_FILE_NO_DATA_CONVERSION,
         VLOG_OPTION_ENUMS,
         DAEMON_OPTION_ENUMS,
         SSL_OPTION_ENUMS,
@@ -1997,6 +2034,8 @@ parse_options(int argc, char *argv[],
         {"active", no_argument, NULL, OPT_ACTIVE},
         {"no-dbs", no_argument, NULL, OPT_NO_DBS},
         {"disable-file-column-diff", no_argument, NULL, OPT_FILE_COLUMN_DIFF},
+        {"disable-file-no-data-conversion", no_argument, NULL,
+         OPT_FILE_NO_DATA_CONVERSION},
         {NULL, 0, NULL, 0},
     };
     char *short_options = ovs_cmdl_long_options_to_short_options(long_options);
@@ -2093,6 +2132,10 @@ parse_options(int argc, char *argv[],
             ovsdb_file_column_diff_disable();
             break;
 
+        case OPT_FILE_NO_DATA_CONVERSION:
+            ovsdb_no_data_conversion_disable();
+            break;
+
         case '?':
             exit(EXIT_FAILURE);
 
